using System.Collections;
using System.Reflection;
using System.Security.Cryptography;

namespace Gep.Api.Util
{
    public class StringUtil
    {   
        public static string ToString(object obj)
        {
            if (obj == null)
            {
                return "null";
            }

            Type type = obj.GetType();
            if (string.Equals("System", type.Namespace))
            {
                return "\"" + obj.ToString() + "\"";
            }

            // class
            string result = "{";

            PropertyInfo[] pis = type.GetProperties();
            for (int i = 0; i < pis.Length; i++)
            {

                PropertyInfo pi = pis[i];
                Type pType = pi.PropertyType;

                MethodInfo getMethod = pi.GetGetMethod();
                object value = getMethod.Invoke(obj, null);
                if (value == null)
                {
                    continue;
                }

                string valueString = "";

                if (string.Equals("System", pType.Namespace))
                {
                    valueString = "\"" + value.ToString() + "\"";
                }
                else if (string.Equals("System.Collections.Generic", pType.Namespace))
                {
                    valueString = List2String(value);
                }
                else
                {
                    valueString = ToString(value);
                }

                if (i != 0)
                {
                    result += ",";
                }
                result += "\"" + pi.Name + "\":" + valueString + "";
            }
            result += "}";

            return result;
        }

        public static string List2String(object listObjects)
        {
            if (listObjects == null)
            {
                return "[]";
            }

            string result = "[";

            IList list = (IList)listObjects;
            for (int i = 0; i < list.Count; i++)
            {
                if (i != 0)
                {
                    result += ",";
                }
                result += ToString(list[i]);
            }
            result += "]";
            return result;
        }
        
        /// <summary>
        /// 获取文件的真实后缀名。目前只支持JPG, GIF, PNG, BMP四种图片文件。
        /// </summary>
        /// <param name="fileData">文件字节流</param>
        /// <returns>JPG, GIF, PNG or null</returns>
        public static string GetFileSuffix(byte[] fileData)
        {
            if (fileData == null || fileData.Length < 10)
            {
                return null;
            }

            if (fileData[0] == 'G' && fileData[1] == 'I' && fileData[2] == 'F')
            {
                return "GIF";
            }
            else if (fileData[1] == 'P' && fileData[2] == 'N' && fileData[3] == 'G')
            {
                return "PNG";
            }
            else if (fileData[6] == 'J' && fileData[7] == 'F' && fileData[8] == 'I' && fileData[9] == 'F')
            {
                return "JPG";
            }
            else if (fileData[0] == 'B' && fileData[1] == 'M')
            {
                return "BMP";
            }
            else
            {
                return null;
            }
        }

        /// <summary>
        /// 获取文件的真实媒体类型。目前只支持JPG, GIF, PNG, BMP四种图片文件。
        /// </summary>
        /// <param name="fileData">文件字节流</param>
        /// <returns>媒体类型</returns>
        public static string GetMimeType(byte[] fileData)
        {
            string suffix = GetFileSuffix(fileData);
            string mimeType;

            switch (suffix)
            {
                case "JPG": mimeType = "image/jpeg"; break;
                case "GIF": mimeType = "image/gif"; break;
                case "PNG": mimeType = "image/png"; break;
                case "BMP": mimeType = "image/bmp"; break;
                default: mimeType = "application/octet-stream"; break;
            }

            return mimeType;
        }
        /// <summary>
        /// 获取文件的MD5值
        /// </summary>
        /// <param name="bytes"></param>
        /// <returns></returns>
        /// <exception cref="Exception"></exception>
         public static string MD5Hash(Byte[] bytes)
        {
            try
            {
                using (var md5 = MD5.Create())
                {
                    byte[] hash = md5.ComputeHash(bytes);
                    string md5Hash = BitConverter.ToString(hash).Replace("-", "").ToLower();

                    Console.WriteLine("MD5 hash of the file: " + md5Hash);
                    return md5Hash;
                }

            }
            catch (Exception ex)
            {
                throw new Exception("Error computing MD5 hash"+ex);
            }
        }
        /// <summary>
        /// 获取当前程序集的目录
        /// </summary>
        /// <returns></returns>
        public static string GetCurrentPath()
        {
            string currentDirectory = Environment.CurrentDirectory;
            //string basePath = Directory.GetParent(Environment.CurrentDirectory).FullName;
            return currentDirectory;
        }
        
        /// <summary>
        /// 生成唯一单号
        /// </summary>
        /// <returns></returns>
        public static string GenerateOrderNumber()
        {
            return Guid.NewGuid().ToString("N").ToUpper();
        }
    }
}